@@ -4,7 +4,7 @@
  * This file is free software; as a special exception the author gives
  * unlimited permission to copy and/or distribute it, with or without
  * modifications, as long as this notice is preserved.
- * 
+ *
  * This file is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY, to the extent permitted by law; without even
  * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
@@ -16,12 +16,12 @@
 
 
 #ifdef USE_SIMPLE_GETTEXT
-# include "../jnlib/w32help.h"
+# include "../common/w32help.h"
 # define _(a) gettext (a)
 # define N_(a) (a)
 #else
 # ifdef HAVE_LOCALE_H
-#  include <locale.h>	
+#  include <locale.h>
 # endif
 # ifdef ENABLE_NLS
 #  include <libintl.h>
