@@ -34,6 +34,15 @@
 #include "options.h"
 #include "main.h"
 #include "i18n.h"
+#include "host2net.h"
+
+
+/* Maximum length of packets to avoid excessive memory allocation.  */
+#define MAX_KEY_PACKET_LENGTH     (256 * 1024)
+#define MAX_UID_PACKET_LENGTH     (  2 * 1024)
+#define MAX_COMMENT_PACKET_LENGTH ( 64 * 1024)
+#define MAX_ATTR_PACKET_LENGTH    ( 16 * 1024*1024)
+
 
 static int mpi_print_mode;
 static int list_mode;
@@ -82,7 +91,7 @@ static unsigned short
 read_16 (IOBUF inp)
 {
   unsigned short a;
-  a = iobuf_get_noeof (inp) << 8;
+  a = (unsigned short)iobuf_get_noeof (inp) << 8;
   a |= iobuf_get_noeof (inp);
   return a;
 }
@@ -92,7 +101,7 @@ static unsigned long
 read_32 (IOBUF inp)
 {
   unsigned long a;
-  a = iobuf_get_noeof (inp) << 24;
+  a = (unsigned long)iobuf_get_noeof (inp) << 24;
   a |= iobuf_get_noeof (inp) << 16;
   a |= iobuf_get_noeof (inp) << 8;
   a |= iobuf_get_noeof (inp);
@@ -478,7 +487,7 @@ parse (IOBUF inp, PACKET * pkt, int onlykeypkts, off_t * retpos,
         }
       else if (c == 255)
         {
-          pktlen = (hdr[hdrlen++] = iobuf_get_noeof (inp)) << 24;
+          pktlen = (unsigned long)(hdr[hdrlen++] = iobuf_get_noeof (inp)) << 24;
           pktlen |= (hdr[hdrlen++] = iobuf_get_noeof (inp)) << 16;
           pktlen |= (hdr[hdrlen++] = iobuf_get_noeof (inp)) << 8;
           if ((c = iobuf_get (inp)) == -1)
@@ -591,7 +600,7 @@ parse (IOBUF inp, PACKET * pkt, int onlykeypkts, off_t * retpos,
                 new_ctb? " new-ctb":"");
 
   pkt->pkttype = pkttype;
-  rc = G10ERR_UNKNOWN_PACKET;	/* default error */
+  rc = GPG_ERR_UNKNOWN_PACKET;	/* default error */
   switch (pkttype)
     {
     case PKT_PUBLIC_KEY:
@@ -657,7 +666,7 @@ parse (IOBUF inp, PACKET * pkt, int onlykeypkts, off_t * retpos,
  leave:
   /* FIXME: Do we leak in case of an error?  */
   if (!rc && iobuf_error (inp))
-    rc = G10ERR_INV_KEYRING;
+    rc = GPG_ERR_INV_KEYRING;
 
   /* FIXME: We use only the error code for now to avoid problems with
      callers which have not been checked to always use gpg_err_code()
@@ -878,7 +887,7 @@ parse_marker (IOBUF inp, int pkttype, unsigned long pktlen)
   if (list_mode)
     es_fputs (":marker packet: [invalid]\n", listfp);
   iobuf_skip_rest (inp, pktlen, 0);
-  return G10ERR_INVALID_PACKET;
+  return GPG_ERR_INV_PACKET;
 }
 
 
@@ -1124,14 +1133,14 @@ dump_sig_subpkt (int hashed, int type, int critical,
     case SIGSUBPKT_SIG_CREATED:
       if (length >= 4)
 	es_fprintf (listfp, "sig created %s",
-                    strtimestamp (buffer_to_u32 (buffer)));
+                    strtimestamp (buf32_to_u32 (buffer)));
       break;
     case SIGSUBPKT_SIG_EXPIRE:
       if (length >= 4)
 	{
-	  if (buffer_to_u32 (buffer))
+	  if (buf32_to_u32 (buffer))
 	    es_fprintf (listfp, "sig expires after %s",
-                        strtimevalue (buffer_to_u32 (buffer)));
+                        strtimevalue (buf32_to_u32 (buffer)));
 	  else
 	    es_fprintf (listfp, "sig does not expire");
 	}
@@ -1164,9 +1173,9 @@ dump_sig_subpkt (int hashed, int type, int critical,
     case SIGSUBPKT_KEY_EXPIRE:
       if (length >= 4)
 	{
-	  if (buffer_to_u32 (buffer))
+	  if (buf32_to_u32 (buffer))
 	    es_fprintf (listfp, "key expires after %s",
-                        strtimevalue (buffer_to_u32 (buffer)));
+                        strtimevalue (buf32_to_u32 (buffer)));
 	  else
 	    es_fprintf (listfp, "key does not expire");
 	}
@@ -1190,8 +1199,8 @@ dump_sig_subpkt (int hashed, int type, int critical,
     case SIGSUBPKT_ISSUER:
       if (length >= 8)
 	es_fprintf (listfp, "issuer key ID %08lX%08lX",
-                    (ulong) buffer_to_u32 (buffer),
-                    (ulong) buffer_to_u32 (buffer + 4));
+                    (ulong) buf32_to_u32 (buffer),
+                    (ulong) buf32_to_u32 (buffer + 4));
       break;
     case SIGSUBPKT_NOTATION:
       {
@@ -1453,8 +1462,7 @@ enum_sig_subpkt (const subpktarea_t * pktbuf, sigsubpkttype_t reqtype,
 	{
 	  if (buflen < 4)
 	    goto too_short;
-	  n = (buffer[0] << 24) | (buffer[1] << 16)
-	    | (buffer[2] << 8) | buffer[3];
+	  n = buf32_to_size_t (buffer);
 	  buffer += 4;
 	  buflen -= 4;
 	}
@@ -1653,7 +1661,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	  log_error ("signature packet: hashed data too long\n");
           if (list_mode)
             es_fputs (":signature packet: [hashed data too long]\n", listfp);
-	  rc = G10ERR_INVALID_PACKET;
+	  rc = GPG_ERR_INV_PACKET;
 	  goto leave;
 	}
       if (n)
@@ -1679,7 +1687,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	  log_error ("signature packet: unhashed data too long\n");
           if (list_mode)
             es_fputs (":signature packet: [unhashed data too long]\n", listfp);
-	  rc = G10ERR_INVALID_PACKET;
+	  rc = GPG_ERR_INV_PACKET;
 	  goto leave;
 	}
       if (n)
@@ -1705,7 +1713,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
       log_error ("packet(%d) too short\n", pkttype);
       if (list_mode)
         es_fputs (":signature packet: [too short]\n", listfp);
-      rc = G10ERR_INVALID_PACKET;
+      rc = GPG_ERR_INV_PACKET;
       goto leave;
     }
 
@@ -1727,7 +1735,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 
       p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_SIG_CREATED, NULL);
       if (p)
-	sig->timestamp = buffer_to_u32 (p);
+	sig->timestamp = buf32_to_u32 (p);
       else if (!(sig->pubkey_algo >= 100 && sig->pubkey_algo <= 110)
 	       && opt.verbose)
 	log_info ("signature packet without timestamp\n");
@@ -1735,16 +1743,16 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
       p = parse_sig_subpkt2 (sig, SIGSUBPKT_ISSUER, NULL);
       if (p)
 	{
-	  sig->keyid[0] = buffer_to_u32 (p);
-	  sig->keyid[1] = buffer_to_u32 (p + 4);
+	  sig->keyid[0] = buf32_to_u32 (p);
+	  sig->keyid[1] = buf32_to_u32 (p + 4);
 	}
       else if (!(sig->pubkey_algo >= 100 && sig->pubkey_algo <= 110)
 	       && opt.verbose)
 	log_info ("signature packet without keyid\n");
 
       p = parse_sig_subpkt (sig->hashed, SIGSUBPKT_SIG_EXPIRE, NULL);
-      if (p && buffer_to_u32 (p))
-	sig->expiredate = sig->timestamp + buffer_to_u32 (p);
+      if (p && buf32_to_u32 (p))
+	sig->expiredate = sig->timestamp + buf32_to_u32 (p);
       if (sig->expiredate && sig->expiredate <= make_timestamp ())
 	sig->flags.expired = 1;
 
@@ -1825,7 +1833,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	  /* We include a limit to avoid too trivial DoS attacks by
 	     having gpg allocate too much memory.  */
 	  log_error ("signature packet: too much data\n");
-	  rc = G10ERR_INVALID_PACKET;
+	  rc = GPG_ERR_INV_PACKET;
 	}
       else
 	{
@@ -1848,7 +1856,7 @@ parse_signature (IOBUF inp, int pkttype, unsigned long pktlen,
 	      es_putc ('\n', listfp);
 	    }
 	  if (!sig->data[i])
-	    rc = G10ERR_INVALID_PACKET;
+	    rc = GPG_ERR_INV_PACKET;
 	}
     }
 
@@ -1954,7 +1962,7 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
   else if (version == 4)
     {
       /* The only supported version.  Use an older gpg
-         versions (i.e. gpg 1.4 to parse v3 packets).  */
+         version (i.e. gpg 1.4) to parse v3 packets.  */
     }
   else if (version == 2 || version == 3)
     {
@@ -1963,7 +1971,7 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
       if (list_mode)
         es_fprintf (listfp, ":key packet: [obsolete version %d]\n", version);
       pk->version = version;
-      err = gpg_error (GPG_ERR_INV_PACKET);
+      err = gpg_error (GPG_ERR_LEGACY_KEY);
       goto leave;
     }
   else
@@ -1983,6 +1991,14 @@ parse_key (IOBUF inp, int pkttype, unsigned long pktlen,
       err = gpg_error (GPG_ERR_INV_PACKET);
       goto leave;
     }
+  else if (pktlen > MAX_KEY_PACKET_LENGTH)
+    {
+      log_error ("packet(%d) too large\n", pkttype);
+      if (list_mode)
+        es_fputs (":key packet: [too larget]\n", listfp);
+      err = gpg_error (GPG_ERR_INV_PACKET);
+      goto leave;
+    }
 
   timestamp = read_32 (inp);
   pktlen -= 4;
@@ -2349,8 +2365,7 @@ parse_attribute_subpkts (PKT_user_id * uid)
 	{
 	  if (buflen < 4)
 	    goto too_short;
-	  n = (buffer[0] << 24) | (buffer[1] << 16)
-	    | (buffer[2] << 8) | buffer[3];
+	  n = buf32_to_size_t (buffer);
 	  buffer += 4;
 	  buflen -= 4;
 	}
@@ -2415,13 +2430,13 @@ parse_user_id (IOBUF inp, int pkttype, unsigned long pktlen, PACKET * packet)
      allocatable, and a very large pktlen could actually cause our
      allocation to wrap around in xmalloc to a small number. */
 
-  if (pktlen > 2048)
+  if (pktlen > MAX_UID_PACKET_LENGTH)
     {
       log_error ("packet(%d) too large\n", pkttype);
       if (list_mode)
         es_fprintf (listfp, ":user ID packet: [too large]\n");
       iobuf_skip_rest (inp, pktlen, 0);
-      return G10ERR_INVALID_PACKET;
+      return GPG_ERR_INV_PACKET;
     }
 
   packet->pkt.user_id = xmalloc_clear (sizeof *packet->pkt.user_id + pktlen);
@@ -2496,13 +2511,13 @@ parse_attribute (IOBUF inp, int pkttype, unsigned long pktlen,
   /* We better cap the size of an attribute packet to make DoS not too
      easy.  16MB should be more then enough for one attribute packet
      (ie. a photo).  */
-  if (pktlen > 16*1024*1024)
+  if (pktlen > MAX_ATTR_PACKET_LENGTH)
     {
       log_error ("packet(%d) too large\n", pkttype);
       if (list_mode)
         es_fprintf (listfp, ":attribute packet: [too large]\n");
       iobuf_skip_rest (inp, pktlen, 0);
-      return G10ERR_INVALID_PACKET;
+      return GPG_ERR_INV_PACKET;
     }
 
 #define EXTRA_UID_NAME_SPACE 71
@@ -2540,14 +2555,14 @@ parse_comment (IOBUF inp, int pkttype, unsigned long pktlen, PACKET * packet)
      overflow in the malloc below.  Comment packets are actually not
      anymore define my OpenPGP and we even stopped to use our
      private comment packet.  */
-  if (pktlen > 65536)
+  if (pktlen > MAX_COMMENT_PACKET_LENGTH)
     {
       log_error ("packet(%d) too large\n", pkttype);
       if (list_mode)
         es_fprintf (listfp, ":%scomment packet: [too large]\n",
                     pkttype == PKT_OLD_COMMENT ? "OpenPGP draft " : "");
       iobuf_skip_rest (inp, pktlen, 0);
-      return G10ERR_INVALID_PACKET;
+      return GPG_ERR_INV_PACKET;
     }
   packet->pkt.comment = xmalloc (sizeof *packet->pkt.comment + pktlen - 1);
   packet->pkt.comment->len = pktlen;
@@ -2580,11 +2595,11 @@ parse_trust (IOBUF inp, int pkttype, unsigned long pktlen, PACKET * pkt)
 
   (void) pkttype;
 
+  pkt->pkt.ring_trust = xmalloc (sizeof *pkt->pkt.ring_trust);
   if (pktlen)
     {
       c = iobuf_get_noeof (inp);
       pktlen--;
-      pkt->pkt.ring_trust = xmalloc (sizeof *pkt->pkt.ring_trust);
       pkt->pkt.ring_trust->trustval = c;
       pkt->pkt.ring_trust->sigcache = 0;
       if (!c && pktlen == 1)
@@ -2603,6 +2618,8 @@ parse_trust (IOBUF inp, int pkttype, unsigned long pktlen, PACKET * pkt)
     }
   else
     {
+      pkt->pkt.ring_trust->trustval = 0;
+      pkt->pkt.ring_trust->sigcache = 0;
       if (list_mode)
 	es_fprintf (listfp, ":trust packet: empty\n");
     }
@@ -2755,7 +2772,7 @@ parse_encrypted (IOBUF inp, int pkttype, unsigned long pktlen,
       log_error ("packet(%d) too short\n", pkttype);
       if (list_mode)
         es_fputs (":encrypted data packet: [too short]\n", listfp);
-      rc = G10ERR_INVALID_PACKET;
+      rc = GPG_ERR_INV_PACKET;
       iobuf_skip_rest (inp, pktlen, partial);
       goto leave;
     }
