@@ -1,5 +1,5 @@
 /* tdbio.c - trust database I/O operations
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2012 Free Software Foundation, Inc.
+ * Copyright (C) 1998-2002, 2012 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -38,7 +38,7 @@
 #include "trustdb.h"
 #include "tdbio.h"
 
-#if defined(HAVE_DOSISH_SYSTEM)
+#if defined(HAVE_DOSISH_SYSTEM) && !defined(ftruncate)
 #define ftruncate chsize
 #endif
 
@@ -48,6 +48,13 @@
 #define MY_O_BINARY  0
 #endif
 
+/* We use ERRNO despite that the cegcc provided open/read/write
+   functions don't set ERRNO - at least show that ERRNO does not make
+   sense.  */
+#ifdef HAVE_W32CE_SYSTEM
+#undef strerror
+#define strerror(a) ("[errno not available]")
+#endif
 
 /****************
  * Yes, this is a very simple implementation. We should really
@@ -86,40 +93,14 @@ struct cmp_xdir_struct {
 
 
 static char *db_name;
-static DOTLOCK lockhandle;
+static dotlock_t lockhandle;
 static int is_locked;
 static int  db_fd = -1;
 static int in_transaction;
 
 static void open_db(void);
 
-static int
-take_write_lock (void)
-{
-  if (!lockhandle)
-    lockhandle = create_dotlock (db_name);
-  if (!lockhandle)
-    log_fatal ( _("can't create lock for `%s'\n"), db_name );
 
-  if (!is_locked)
-    {
-      if (make_dotlock (lockhandle, -1) )
-        log_fatal ( _("can't lock `%s'\n"), db_name );
-      else
-        is_locked = 1;
-      return 0;
-    }
-  else
-    return 1;
-}
-
-static void
-release_write_lock (void)
-{
-  if (!opt.lock_once)
-    if (!release_dotlock (lockhandle))
-      is_locked = 0;
-}
 
 /*************************************
  ************* record cache **********
@@ -269,13 +250,18 @@ put_record_into_cache( ulong recno, const char *data )
 	    return 0;
 	}
 	log_info(_("trustdb transaction too large\n"));
-	return G10ERR_RESOURCE_LIMIT;
+	return GPG_ERR_RESOURCE_LIMIT;
     }
     if( dirty_count ) {
 	int n = dirty_count / 5; /* discard some dirty entries */
 	if( !n )
 	    n = 1;
-        take_write_lock ();
+	if( !is_locked ) {
+	    if( dotlock_take( lockhandle, -1 ) )
+		log_fatal("can't acquire lock - giving up\n");
+	    else
+		is_locked = 1;
+	}
 	for( unused = NULL, r = cache_list; r; r = r->next ) {
 	    if( r->flags.used && r->flags.dirty ) {
 		int rc = write_cache_item( r );
@@ -289,7 +275,10 @@ put_record_into_cache( ulong recno, const char *data )
 		    break;
 	    }
 	}
-        release_write_lock ();
+	if( !opt.lock_once ) {
+	    if( !dotlock_release( lockhandle ) )
+		is_locked = 0;
+	}
 	assert( unused );
 	r = unused;
 	r->flags.used = 1;
@@ -328,9 +317,13 @@ tdbio_sync()
     if( !cache_is_dirty )
 	return 0;
 
-    if (!take_write_lock ())
-        did_lock = 1;
-
+    if( !is_locked ) {
+	if( dotlock_take( lockhandle, -1 ) )
+	    log_fatal("can't acquire lock - giving up\n");
+	else
+	    is_locked = 1;
+	did_lock = 1;
+    }
     for( r = cache_list; r; r = r->next ) {
 	if( r->flags.used && r->flags.dirty ) {
 	    int rc = write_cache_item( r );
@@ -339,8 +332,10 @@ tdbio_sync()
 	}
     }
     cache_is_dirty = 0;
-    if (did_lock)
-        release_write_lock ();
+    if( did_lock && !opt.lock_once ) {
+	if( !dotlock_release (lockhandle) )
+	    is_locked = 0;
+    }
 
     return 0;
 }
@@ -377,12 +372,20 @@ tdbio_end_transaction()
 
     if( !in_transaction )
 	log_bug("tdbio: no active transaction\n");
-    take_write_lock ();
+    if( !is_locked ) {
+	if( dotlock_take( lockhandle, -1 ) )
+	    log_fatal("can't acquire lock - giving up\n");
+	else
+	    is_locked = 1;
+    }
     block_all_signals();
     in_transaction = 0;
     rc = tdbio_sync();
     unblock_all_signals();
-    release_write_lock ();
+    if( !opt.lock_once ) {
+	if( !dotlock_release (lockhandle) )
+	    is_locked = 0;
+    }
     return rc;
 }
 
@@ -420,7 +423,7 @@ static void
 cleanup(void)
 {
     if( is_locked ) {
-	if( !release_dotlock(lockhandle) )
+	if( !dotlock_release (lockhandle) )
 	    is_locked = 0;
     }
 }
@@ -480,7 +483,6 @@ int
 tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
 {
     char *fname;
-    struct stat statbuf;
     static int initialized = 0;
 
     if( !initialized ) {
@@ -491,7 +493,7 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
     *r_nofile = 0;
 
     if(new_dbname==NULL)
-      fname=make_filename(opt.homedir,"trustdb" EXTSEP_S "gpg", NULL);
+      fname=make_filename(opt.homedir,"trustdb" EXTSEP_S GPGEXT_GPG, NULL);
     else if (*new_dbname != DIRSEP_C )
       {
 	if (strchr(new_dbname, DIRSEP_C) )
@@ -502,25 +504,19 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
     else
       fname = xstrdup (new_dbname);
 
-    xfree (db_name);
-    db_name = fname;
-
-    /*
-     * Quick check for (likely) case where there is trustdb.gpg
-     * already.  This check is not required in theory, but it helps in
-     * practice, avoiding costly operations of preparing and taking
-     * the lock.
-     */
-    if (stat (fname, &statbuf) == 0 && statbuf.st_size > 0)
-      /* OK, we have the valid trustdb.gpg already.  */
-      return 0;
-
-    take_write_lock ();
-
     if( access( fname, R_OK ) ) {
-        if( errno != ENOENT )
-            log_fatal( _("can't access `%s': %s\n"), fname, strerror(errno) );
-
+#ifdef HAVE_W32CE_SYSTEM
+      /* We know how the cegcc implementation of access works ;-). */
+      if (GetLastError () == ERROR_FILE_NOT_FOUND)
+        gpg_err_set_errno (ENOENT);
+      else
+        gpg_err_set_errno (EIO);
+#endif /*HAVE_W32CE_SYSTEM*/
+	if( errno != ENOENT ) {
+	    log_error( _("can't access '%s': %s\n"), fname, strerror(errno) );
+	    xfree(fname);
+	    return GPG_ERR_TRUSTDB;
+	}
 	if (!create)
           *r_nofile = 1;
         else {
@@ -550,35 +546,56 @@ tdbio_set_dbname( const char *new_dbname, int create, int *r_nofile)
 	    }
 	    *p = save_slash;
 
+	    xfree(db_name);
+	    db_name = fname;
+#ifdef __riscos__
+	    if( !lockhandle )
+              lockhandle = dotlock_create (db_name, 0);
+	    if( !lockhandle )
+		log_fatal( _("can't create lock for '%s'\n"), db_name );
+            if( dotlock_make (lockhandle, -1) )
+                log_fatal( _("can't lock '%s'\n"), db_name );
+#endif /* __riscos__ */
 	    oldmask=umask(077);
             if (is_secured_filename (fname)) {
                 fp = NULL;
-                errno = EPERM;
+                gpg_err_set_errno (EPERM);
             }
             else
                 fp =fopen( fname, "wb" );
 	    umask(oldmask);
 	    if( !fp )
-		log_fatal( _("can't create `%s': %s\n"), fname, strerror(errno) );
+		log_fatal (_("can't create '%s': %s\n"),
+                           fname, strerror (errno));
 	    fclose(fp);
 	    db_fd = open( db_name, O_RDWR | MY_O_BINARY );
 	    if( db_fd == -1 )
-		log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
+		log_fatal (_("can't open '%s': %s\n"),
+                           db_name, strerror (errno));
+
+#ifndef __riscos__
+	    if( !lockhandle )
+              lockhandle = dotlock_create (db_name, 0);
+	    if( !lockhandle )
+		log_fatal( _("can't create lock for '%s'\n"), db_name );
+#endif /* !__riscos__ */
 
             rc = create_version_record ();
 	    if( rc )
 		log_fatal( _("%s: failed to create version record: %s"),
-						   fname, g10_errstr(rc));
+						   fname, gpg_strerror (rc));
 	    /* and read again to check that we are okay */
 	    if( tdbio_read_record( 0, &rec, RECTYPE_VER ) )
 		log_fatal( _("%s: invalid trustdb created\n"), db_name );
 
 	    if( !opt.quiet )
 		log_info(_("%s: trustdb created\n"), db_name);
+
+	    return 0;
 	}
     }
-
-    release_write_lock ();
+    xfree(db_name);
+    db_name = fname;
     return 0;
 }
 
@@ -598,6 +615,30 @@ open_db()
 
   assert( db_fd == -1 );
 
+  if (!lockhandle )
+    lockhandle = dotlock_create (db_name, 0);
+  if (!lockhandle )
+    log_fatal( _("can't create lock for '%s'\n"), db_name );
+#ifdef __riscos__
+  if (dotlock_take (lockhandle, -1) )
+    log_fatal( _("can't lock '%s'\n"), db_name );
+#endif /* __riscos__ */
+#ifdef HAVE_W32CE_SYSTEM
+  {
+    DWORD prevrc = 0;
+    wchar_t *wname = utf8_to_wchar (db_name);
+    if (wname)
+      {
+        db_fd = (int)CreateFile (wname, GENERIC_READ|GENERIC_WRITE,
+                                 FILE_SHARE_READ|FILE_SHARE_WRITE, NULL,
+                                 OPEN_EXISTING, 0, NULL);
+        xfree (wname);
+      }
+    if (db_fd == -1)
+      log_fatal ("can't open '%s': %d, %d\n", db_name,
+                 (int)prevrc, (int)GetLastError ());
+  }
+#else /*!HAVE_W32CE_SYSTEM*/
   db_fd = open (db_name, O_RDWR | MY_O_BINARY );
   if (db_fd == -1 && (errno == EACCES
 #ifdef EROFS
@@ -605,12 +646,14 @@ open_db()
 #endif
                       )
       ) {
+      /* Take care of read-only trustdbs.  */
       db_fd = open (db_name, O_RDONLY | MY_O_BINARY );
       if (db_fd != -1 && !opt.quiet)
-          log_info (_("NOTE: trustdb not writable\n"));
+          log_info (_("Note: trustdb not writable\n"));
   }
   if ( db_fd == -1 )
-    log_fatal( _("can't open `%s': %s\n"), db_name, strerror(errno) );
+    log_fatal( _("can't open '%s': %s\n"), db_name, strerror(errno) );
+#endif /*!HAVE_W32CE_SYSTEM*/
   register_secured_file (db_name);
 
   /* Read the version record. */
@@ -648,7 +691,7 @@ create_hashtable( TRUSTREC *vr, int type )
 	 rc = tdbio_write_record( &rec );
 	 if( rc )
 	     log_fatal( _("%s: failed to create hashtable: %s\n"),
-					db_name, g10_errstr(rc));
+					db_name, gpg_strerror (rc));
     }
     /* update the version record */
     rc = tdbio_write_record( vr );
@@ -656,7 +699,7 @@ create_hashtable( TRUSTREC *vr, int type )
 	rc = tdbio_sync();
     if( rc )
 	log_fatal( _("%s: error updating version record: %s\n"),
-						  db_name, g10_errstr(rc));
+						  db_name, gpg_strerror (rc));
 }
 
 
@@ -673,7 +716,7 @@ tdbio_db_matches_options()
       rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
       if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-		   db_name, g10_errstr(rc) );
+		   db_name, gpg_strerror (rc) );
 
       yes_no = vr.r.ver.marginals == opt.marginals_needed
 	&& vr.r.ver.completes == opt.completes_needed
@@ -694,7 +737,7 @@ tdbio_read_model(void)
   rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
   if( rc )
     log_fatal( _("%s: error reading version record: %s\n"),
-	       db_name, g10_errstr(rc) );
+	       db_name, gpg_strerror (rc) );
   return vr.r.ver.trust_model;
 }
 
@@ -710,7 +753,7 @@ tdbio_read_nextcheck ()
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-						    db_name, g10_errstr(rc) );
+						    db_name, gpg_strerror (rc));
     return vr.r.ver.nextcheck;
 }
 
@@ -724,7 +767,7 @@ tdbio_write_nextcheck (ulong stamp)
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
 
     if (vr.r.ver.nextcheck == stamp)
       return 0;
@@ -733,7 +776,7 @@ tdbio_write_nextcheck (ulong stamp)
     rc = tdbio_write_record( &vr );
     if( rc )
 	log_fatal( _("%s: error writing version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
     return 1;
 }
 
@@ -754,7 +797,7 @@ get_trusthashrec(void)
 	rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
 	if( rc )
 	    log_fatal( _("%s: error reading version record: %s\n"),
-					    db_name, g10_errstr(rc) );
+					    db_name, gpg_strerror (rc) );
 	if( !vr.r.ver.trusthashtbl )
 	    create_hashtable( &vr, 0 );
 
@@ -785,7 +828,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error("upd_hashtable: read failed: %s\n",	g10_errstr(rc) );
+	log_error("upd_hashtable: read failed: %s\n",	gpg_strerror (rc) );
 	return rc;
     }
 
@@ -795,7 +838,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	rc = tdbio_write_record( &rec );
 	if( rc ) {
 	    log_error("upd_hashtable: write htbl failed: %s\n",
-							    g10_errstr(rc) );
+							    gpg_strerror (rc) );
 	    return rc;
 	}
     }
@@ -804,7 +847,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	rc = tdbio_read_record( item, &rec, 0 );
 	if( rc ) {
 	    log_error( "upd_hashtable: read item failed: %s\n",
-							    g10_errstr(rc) );
+							    gpg_strerror (rc) );
 	    return rc;
 	}
 
@@ -813,7 +856,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	    level++;
 	    if( level >= keylen ) {
 		log_error( "hashtable has invalid indirections.\n");
-		return G10ERR_TRUSTDB;
+		return GPG_ERR_TRUSTDB;
 	    }
 	    goto next_level;
 	}
@@ -829,8 +872,8 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_read_record( rec.r.hlst.next,
 						       &rec, RECTYPE_HLST);
 		    if( rc ) {
-			log_error( "upd_hashtable: read hlst failed: %s\n",
-							     g10_errstr(rc) );
+			log_error ("upd_hashtable: read hlst failed: %s\n",
+                                   gpg_strerror (rc) );
 			return rc;
 		    }
 		}
@@ -844,8 +887,8 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 			rec.r.hlst.rnum[i] = newrecnum;
 			rc = tdbio_write_record( &rec );
 			if( rc )
-			    log_error( "upd_hashtable: write hlst failed: %s\n",
-							      g10_errstr(rc) );
+			    log_error ("upd_hashtable: write hlst failed: %s\n",
+                                       gpg_strerror (rc));
 			return rc; /* done */
 		    }
 		}
@@ -853,8 +896,8 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_read_record( rec.r.hlst.next,
 						      &rec, RECTYPE_HLST );
 		    if( rc ) {
-			log_error( "upd_hashtable: read hlst failed: %s\n",
-							     g10_errstr(rc) );
+			log_error ("upd_hashtable: read hlst failed: %s\n",
+                                   gpg_strerror (rc));
 			return rc;
 		    }
 		}
@@ -863,7 +906,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_write_record( &rec );
 		    if( rc ) {
 			log_error( "upd_hashtable: write hlst failed: %s\n",
-							  g10_errstr(rc) );
+							  gpg_strerror (rc) );
 			return rc;
 		    }
 		    memset( &rec, 0, sizeof rec );
@@ -873,7 +916,7 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 		    rc = tdbio_write_record( &rec );
 		    if( rc )
 			log_error( "upd_hashtable: write ext hlst failed: %s\n",
-							  g10_errstr(rc) );
+							  gpg_strerror (rc) );
 		    return rc; /* done */
 		}
 	    } /* end loop over hlst slots */
@@ -891,22 +934,22 @@ upd_hashtable( ulong table, byte *key, int keylen, ulong newrecnum )
 	    rc = tdbio_write_record( &rec );
 	    if( rc ) {
 		log_error( "upd_hashtable: write new hlst failed: %s\n",
-						  g10_errstr(rc) );
+						  gpg_strerror (rc) );
 		return rc;
 	    }
 	    /* update the hashtable record */
 	    lastrec.r.htbl.item[msb % ITEMS_PER_HTBL_RECORD] = rec.recnum;
 	    rc = tdbio_write_record( &lastrec );
 	    if( rc )
-		log_error( "upd_hashtable: update htbl failed: %s\n",
-							     g10_errstr(rc) );
+		log_error ("upd_hashtable: update htbl failed: %s\n",
+                           gpg_strerror (rc));
 	    return rc; /* ready */
 	}
 	else {
 	    log_error( "hashtbl %lu: %lu/%d points to an invalid record %lu\n",
 		       table, hashrec, (msb % ITEMS_PER_HTBL_RECORD), item);
 	    list_trustdb(NULL);
-	    return G10ERR_TRUSTDB;
+	    return GPG_ERR_TRUSTDB;
 	}
     }
 
@@ -934,7 +977,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
     rc = tdbio_read_record( hashrec, &rec, RECTYPE_HTBL );
     if( rc ) {
 	log_error("drop_from_hashtable: read failed: %s\n",
-							g10_errstr(rc) );
+							gpg_strerror (rc) );
 	return rc;
     }
 
@@ -947,14 +990,14 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 	rc = tdbio_write_record( &rec );
 	if( rc )
 	    log_error("drop_from_hashtable: write htbl failed: %s\n",
-							    g10_errstr(rc) );
+							    gpg_strerror (rc) );
 	return rc;
     }
 
     rc = tdbio_read_record( item, &rec, 0 );
     if( rc ) {
 	log_error( "drop_from_hashtable: read item failed: %s\n",
-							g10_errstr(rc) );
+							gpg_strerror (rc) );
 	return rc;
     }
 
@@ -963,7 +1006,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 	level++;
 	if( level >= keylen ) {
 	    log_error( "hashtable has invalid indirections.\n");
-	    return G10ERR_TRUSTDB;
+	    return GPG_ERR_TRUSTDB;
 	}
 	goto next_level;
     }
@@ -975,8 +1018,8 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 		    rec.r.hlst.rnum[i] = 0; /* drop */
 		    rc = tdbio_write_record( &rec );
 		    if( rc )
-			log_error("drop_from_hashtable: write htbl failed: %s\n",
-									g10_errstr(rc) );
+                      log_error("drop_from_hashtable: write htbl failed: %s\n",
+                                gpg_strerror (rc));
 		    return rc;
 		}
 	    }
@@ -985,7 +1028,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 						   &rec, RECTYPE_HLST);
 		if( rc ) {
 		    log_error( "drop_from_hashtable: read hlst failed: %s\n",
-							 g10_errstr(rc) );
+							 gpg_strerror (rc) );
 		    return rc;
 		}
 	    }
@@ -996,7 +1039,7 @@ drop_from_hashtable( ulong table, byte *key, int keylen, ulong recnum )
 
     log_error( "hashtbl %lu: %lu/%d points to wrong record %lu\n",
 		    table, hashrec, (msb % ITEMS_PER_HTBL_RECORD), item);
-    return G10ERR_TRUSTDB;
+    return GPG_ERR_TRUSTDB;
 }
 
 
@@ -1022,7 +1065,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
     hashrec += msb / ITEMS_PER_HTBL_RECORD;
     rc = tdbio_read_record( hashrec, rec, RECTYPE_HTBL );
     if( rc ) {
-	log_error("lookup_hashtable failed: %s\n", g10_errstr(rc) );
+	log_error("lookup_hashtable failed: %s\n", gpg_strerror (rc) );
 	return rc;
     }
 
@@ -1032,7 +1075,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 
     rc = tdbio_read_record( item, rec, 0 );
     if( rc ) {
-	log_error( "hashtable read failed: %s\n", g10_errstr(rc) );
+	log_error( "hashtable read failed: %s\n", gpg_strerror (rc) );
 	return rc;
     }
     if( rec->rectype == RECTYPE_HTBL ) {
@@ -1040,7 +1083,7 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 	level++;
 	if( level >= keylen ) {
 	    log_error("hashtable has invalid indirections\n");
-	    return G10ERR_TRUSTDB;
+	    return GPG_ERR_TRUSTDB;
 	}
 	goto next_level;
     }
@@ -1054,8 +1097,8 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 
 		    rc = tdbio_read_record( rec->r.hlst.rnum[i], &tmp, 0 );
 		    if( rc ) {
-			log_error( "lookup_hashtable: read item failed: %s\n",
-							      g10_errstr(rc) );
+			log_error ("lookup_hashtable: read item failed: %s\n",
+                                   gpg_strerror (rc));
 			return rc;
 		    }
 		    if( (*cmpfnc)( cmpdata, &tmp ) ) {
@@ -1067,8 +1110,8 @@ lookup_hashtable( ulong table, const byte *key, size_t keylen,
 	    if( rec->r.hlst.next ) {
 		rc = tdbio_read_record( rec->r.hlst.next, rec, RECTYPE_HLST );
 		if( rc ) {
-		    log_error( "lookup_hashtable: read hlst failed: %s\n",
-							 g10_errstr(rc) );
+		    log_error ("lookup_hashtable: read hlst failed: %s\n",
+                               gpg_strerror (rc) );
 		    return rc;
 		}
 	    }
@@ -1202,7 +1245,7 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
       case 0:  /* unused (free) record */
 	break;
       case RECTYPE_VER: /* version record */
-	if( memcmp(buf+1, "gpg", 3 ) ) {
+	if( memcmp(buf+1, GPGEXT_GPG, 3 ) ) {
 	    log_error( _("%s: not a trustdb file\n"), db_name );
 	    err = gpg_error (GPG_ERR_TRUSTDB);
 	}
@@ -1214,13 +1257,13 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	rec->r.ver.trust_model = *p++;
 	rec->r.ver.min_cert_level = *p++;
 	p += 2;
-	rec->r.ver.created  = buf32_to_ulong (p); p += 4;
-	rec->r.ver.nextcheck = buf32_to_ulong (p); p += 4;
+	rec->r.ver.created  = buf32_to_ulong(p); p += 4;
+	rec->r.ver.nextcheck = buf32_to_ulong(p); p += 4;
 	p += 4;
 	p += 4;
-	rec->r.ver.firstfree =buf32_to_ulong (p); p += 4;
+	rec->r.ver.firstfree =buf32_to_ulong(p); p += 4;
 	p += 4;
-	rec->r.ver.trusthashtbl =buf32_to_ulong (p); p += 4;
+	rec->r.ver.trusthashtbl =buf32_to_ulong(p); p += 4;
 	if( recnum ) {
 	    log_error( _("%s: version record with recnum %lu\n"), db_name,
 							     (ulong)recnum );
@@ -1233,17 +1276,17 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
 	}
 	break;
       case RECTYPE_FREE:
-	rec->r.free.next  = buf32_to_ulong (p); p += 4;
+	rec->r.free.next  = buf32_to_ulong(p); p += 4;
 	break;
       case RECTYPE_HTBL:
 	for(i=0; i < ITEMS_PER_HTBL_RECORD; i++ ) {
-	    rec->r.htbl.item[i] = buf32_to_ulong (p); p += 4;
+	    rec->r.htbl.item[i] = buf32_to_ulong(p); p += 4;
 	}
 	break;
       case RECTYPE_HLST:
-	rec->r.hlst.next = buf32_to_ulong (p); p += 4;
+	rec->r.hlst.next = buf32_to_ulong(p); p += 4;
 	for(i=0; i < ITEMS_PER_HLST_RECORD; i++ ) {
-	    rec->r.hlst.rnum[i] = buf32_to_ulong (p); p += 4;
+	    rec->r.hlst.rnum[i] = buf32_to_ulong(p); p += 4;
 	}
 	break;
       case RECTYPE_TRUST:
@@ -1252,12 +1295,12 @@ tdbio_read_record( ulong recnum, TRUSTREC *rec, int expected )
         rec->r.trust.depth = *p++;
         rec->r.trust.min_ownertrust = *p++;
         p++;
-	rec->r.trust.validlist = buf32_to_ulong (p); p += 4;
+	rec->r.trust.validlist = buf32_to_ulong(p); p += 4;
 	break;
       case RECTYPE_VALID:
 	memcpy( rec->r.valid.namehash, p, 20); p+=20;
         rec->r.valid.validity = *p++;
-	rec->r.valid.next = buf32_to_ulong (p); p += 4;
+	rec->r.valid.next = buf32_to_ulong(p); p += 4;
 	rec->r.valid.full_count = *p++;
 	rec->r.valid.marginal_count = *p++;
 	break;
@@ -1294,7 +1337,7 @@ tdbio_write_record( TRUSTREC *rec )
       case RECTYPE_VER: /* version record */
 	if( recnum )
 	    BUG();
-	memcpy(p-1, "gpg", 3 ); p += 2;
+	memcpy(p-1, GPGEXT_GPG, 3 ); p += 2;
 	*p++ = rec->r.ver.version;
 	*p++ = rec->r.ver.marginals;
 	*p++ = rec->r.ver.completes;
@@ -1381,7 +1424,7 @@ tdbio_delete_record( ulong recnum )
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-				       db_name, g10_errstr(rc) );
+				       db_name, gpg_strerror (rc) );
 
     rec.recnum = recnum;
     rec.rectype = RECTYPE_FREE;
@@ -1408,21 +1451,21 @@ tdbio_new_recnum()
     rc = tdbio_read_record( 0, &vr, RECTYPE_VER );
     if( rc )
 	log_fatal( _("%s: error reading version record: %s\n"),
-					     db_name, g10_errstr(rc) );
+					     db_name, gpg_strerror (rc) );
     if( vr.r.ver.firstfree ) {
 	recnum = vr.r.ver.firstfree;
 	rc = tdbio_read_record( recnum, &rec, RECTYPE_FREE );
 	if( rc ) {
 	    log_error( _("%s: error reading free record: %s\n"),
-						  db_name,  g10_errstr(rc) );
+						  db_name,  gpg_strerror (rc) );
 	    return rc;
 	}
 	/* update dir record */
 	vr.r.ver.firstfree = rec.r.free.next;
 	rc = tdbio_write_record( &vr );
 	if( rc ) {
-	    log_error( _("%s: error writing dir record: %s\n"),
-						     db_name, g10_errstr(rc) );
+	    log_error (_("%s: error writing dir record: %s\n"),
+                       db_name, gpg_strerror (rc));
 	    return rc;
 	}
 	/*zero out the new record */
@@ -1432,7 +1475,7 @@ tdbio_new_recnum()
 	rc = tdbio_write_record( &rec );
 	if( rc )
 	    log_fatal(_("%s: failed to zero a record: %s\n"),
-				       db_name, g10_errstr(rc));
+				       db_name, gpg_strerror (rc));
     }
     else { /* not found, append a new record */
 	offset = lseek( db_fd, 0, SEEK_END );
@@ -1462,7 +1505,7 @@ tdbio_new_recnum()
 
 	if( rc )
 	    log_fatal(_("%s: failed to append a record: %s\n"),
-				    db_name,	g10_errstr(rc));
+				    db_name,	gpg_strerror (rc));
     }
     return recnum ;
 }
@@ -1508,4 +1551,3 @@ tdbio_invalid(void)
   how_to_fix_the_trustdb ();
   g10_exit (2);
 }
-
